#include "space.h"
#include <QGraphicsItem>
#include <QDebug>
#include <cmath>

Space::Space(int width, int height, bool boundaryPeriodic) {
    _width = width;
    _height = height;
    _boundaryPeriodic = boundaryPeriodic;
    struct CellLocation loc;
    for(int x = 0; x < _width; x++) {
        loc.x = x;
        _cells.append(QVector<Cell>());
        for(int y = 0; y < _height; y++) {
            loc.y = y;
            _cells[x].append(Cell());
            _emptyCells.append(loc);
        }
    }
}

Space::~Space() {
    _freeCells(_cells);
}

void Space::draw(QGraphicsScene *scene) {
    QColor color;
    QGraphicsRectItem * item;

    scene->clear();
    for(int x = 0; x < _width; x++) {
        for(int y = 0; y < _height; y++) {
            if (_cells[x][y].isEmpty()) {
                color = QColor(0xFF, 0xFF, 0xFF);
            }
            else if (_cells[x][y].onBorder()) {
                color = QColor(0, 0, 0);
            }
            else if (_cells[x][y].isLocked()) {
                color = QColor(0, 0, 0);
            }
            else {
                int id = _cells[x][y].getGrainId();
                color = QColor((Qt::GlobalColor)(5 + (id % 10)));
                if (id > 10)
                    color.setAlpha(0x80);
            }

            item = scene->addRect(x * CELL_WIDTH, y * CELL_HEIGHT, CELL_WIDTH, CELL_HEIGHT, QPen(Qt::black), QBrush(color));
        }
    }
}

bool Space::setCellId(int x, int y, int id) {
    if (!_cells[x][y].isEmpty())
        return false;

    if (_cells[x][y].isLocked())
        return false;

    _cells[x][y].setGrainId(id);
    return true;
}

void Space::setSimulation(Simulation * sim) {
    _simulation = sim;
}

bool Space::simulationStep() {
    QVector<QVector<Cell> > copy;

    _copyCellsTo(copy);

    _simulation->step(copy, _cells, _boundaryPeriodic, _emptyCells);

    _freeCells(copy);

    return _emptyCells.isEmpty();
}

bool Space::boundaryIdPeriodic() {
    return _boundaryPeriodic;
}

void Space::addInclusion(int x, int y, Inclusion inc) {
    int tx, ty;
    double tmp;
    int d = inc.dimension;

    if (inc.isRounded)
        d *= 2;

    tx = x - (d / 2);
    ty = y - (d / 2);

    if (tx < 0)
        tx = 0;
    if (ty < 0)
        ty = 0;

    qDebug() << "tx =" << tx << "ty =" << ty;

    for(int ix = 0; ix < d; ix++) {
        for(int iy = 0; iy < d; iy++) {
            if ((tx + ix >= _cells.count()) || (ty + iy >= _cells[tx + ix].count()))
                continue;

            if (inc.isRounded) {
                tmp = (tx + ix - x) * (tx + ix - x) + (ty + iy - y) * (ty + iy - y);
                tmp = sqrt(tmp);
                qDebug() << "tmp =" << tmp << "d =" << inc.dimension;
                if (ceil(tmp) < inc.dimension) {
                    _cells[tx + ix][ty + iy].setGrainId(-1);
                    _cells[tx + ix][ty + iy].lock();
                }
            }
            else {
                _cells[tx + ix][ty + iy].setGrainId(-1);
                _cells[tx + ix][ty + iy].lock();
            }
        }
    }
}

int Space::width() {
    return _width;
}

int Space::height() {
    return _height;
}

bool Space::isCellOnBorder(int x, int y) {
    if ((x < 0) || (y < 0) || (x >= _width) || (y >= _height))
        return false;

    return _cells[x][y].onBorder();
}

bool Space::simulationFinished() {
    return _emptyCells.isEmpty();
}

void Space::_copyCellsTo(QVector<QVector<Cell> > &dest) {
    for(int x = 0; x < _width; x++) {
        dest.append(QVector<Cell>());

        for(int y = 0; y < _height; y++) {
            dest[x].append(Cell(_cells[x][y]));
        }
    }
}

void Space::_freeCells(QVector<QVector<Cell> > &cells) {
    for(int x = 0; x < _width; x++) {
        cells[x].clear();
    }

    cells.clear();
}
